##
##
## Project: GEOPAL
##
##
## Script:  functions
##
##
## Author:  Jonathan Schoenenberg
##
##
## Date:    2021-05-20
##
##
## Contact: j.schoenenberg@uni-koeln.de
##
##
##
### getdata:   read data from folders ---- 

getdata <- function(path){
  
  file_paths <- fs::dir_ls(path)
  
  dat.list <- list()
  
  
  # iterate through files in folder
  for (i in seq_along(file_paths)){
    dat.list[[i]] <- read.csv(file = file_paths[[i]], fileEncoding = "UTF-8")
  }
  
  
  # name the different elements, sub() to select file name from path.
  dat.list <- setNames(dat.list, sub(".*/", "", file_paths))

  
  
  # rename data in list for further processing
  dat.names <- names(dat.list)
  
  
  dat.names <- sub("-back", ".b", dat.names)
  dat.names <- gsub("_3035.*|_cat-|eu_|_impless", "", dat.names)
  dat.names <- sub("3600", "1", dat.names)
  dat.names <- sub("7200", "2", dat.names)
  dat.names <- sub("10800", "3", dat.names)
  dat.names <- sub("14400", "4", dat.names)
  
  names(dat.list) <- dat.names
  
  
  if (sub(".*/", "", path) == "impedance") {
    impedance <- TRUE
  } else {
    impedance <- FALSE
    
    # rename and copy background data for different periods, as they are identical without impedance
    # temporary store the background data
    tmp <- dat.list[which(names(dat.list)%like% "%na%")]

    # delete data in original list
    dat.list[which(names(dat.list)%like% "%na%")] <- NULL
    
    # iterate through periods, rename data and append to list 
    for (i in 1:length(color.scheme)){
      names(tmp) <- sub("..", names(color.scheme)[i], names(tmp))
      names(tmp) <- sub(".*\\.", paste0(names(color.scheme)[i],"."), names(tmp))
      dat.list <- c(dat.list, tmp)
    }
    
    
  }
  
  data <- list('datlist' = dat.list, 'impedance' = impedance)
  
  return(data)
}



### sqm:       pixel to Square km ---- 

# multiplying only the LFC-columns with the Square km of one raster cell (250 x 250 m = 0.0625 km2)
sqm <- function(data){
  data[,c(which(colnames(data) %like% "LFC%"==TRUE))] <- data[,c(which(colnames(data) %like% "LFC%"==TRUE))]*0.0625
  
  return(data)
}



### del.nodat: delete no-data LFC ---- 

del.nodat<- function(data){
  if((sum(data$LFC_NODATA)==0)==T){
    data$LFC_NODATA <- NULL
  } else{
    print("at least 1 catchment calculation went wrong, check ya data!")
  }
  return(data)
}



### tabgen:    generate table for Chi-square test ---- 

tabgen <- function(class.dat, col){
  
  # define isochrones of sites
  chi.dat.sit.names <- unique(gsub(".b","" , names(class.dat))) 
  
  # define isochrones of background distribution
  chi.dat.back.names <- names(class.dat[which(names(class.dat) %like% "%.b%")]) 
  
  
  chi.dat <- as.list(chi.dat.sit.names)
  names(chi.dat) <- chi.dat.sit.names
  
  for(i in 1:length(chi.dat)){
    
    back <- class.dat[[chi.dat.back.names[i]]]
    sit <- class.dat[[chi.dat.sit.names[i]]]
    
    # defines which data to extract, for example "Amount" or "share".
    col.index <- which(colnames(back) == col) 
    chi.dat[[i]] <- data.frame(Background = back[, col.index], Sites = sit[, col.index])
  }
  
  return(chi.dat)
}



### class.chi: sum classes by expectation value (Chi-Square Test), return classified data ---- 

class.chi <- function(data, exp.val){
  
  # delete empty classes, if there are some
  empty.c <- as.vector(which(rowSums(data) == 0))
  if(is.na(empty.c[1]) == FALSE){
    data <- data[-empty.c,]
  } 
  
  xsq_data <- chisq.test(data)
  exp <- data.frame(cbind(xsq_data$expected, index = rep(0,nrow(data))))
  
  # iterate as long as any expectation value is below exp.val
  while(length(unique(unlist(apply(exp[1:2], 2, function(x) which(x < exp.val))))) > 0){
    
    # creating an index with rows containing expectation values < exp.val
    index <- unique(unlist(apply(exp[1:2], 2, function(x) which(x < exp.val)))) 
    
    # set index
    exp$index[index] <- 1
    
    # iterate
    for(i in 1:(length(exp$index))){
      
      if((exp$index[i] > 0) & (i < nrow(exp))){
        exp[i,] <- exp[i,]+exp[i+1,]
        exp <- exp[-(i+1),]
        data[i,] <- data[i,] + data[i+1,]
        data <- data[-(i+1),]
        
        # if the last class must be combined, sum with previous one 
      } else if ((exp$index[i] > 0) & (i == nrow(exp))){ 
        exp[i-1,] <- exp[i-1,]+exp[i,]
        exp <- exp[-i,]
        data[i-1,] <- data[i-1,]+data[i,]
        data <- data[-i,]
      }
    }
    
    # clear index
    exp$index <- 0
  }
  
  return(data)
  
}



### con.dat:   convert data from list to data.frame ---- 

con.dat <- function(data,variable){
  data <- do.call(rbind.data.frame, data)
  
  data$PER <- substr(rownames(data),1,3)
  data$CLASS <- as.integer(sub(".*\\.","",rownames(data)))
  data$TYPE <- rep(NA,length(data$CLASS))
  data$TYPE[which(rownames(data) %like% "%.b%")] <- "Back"
  data$TYPE[which(is.na(data$TYPE))] <- "Site"
  data$ISO <- as.numeric(sub("\\D*(\\d{1}).*", "\\1", rownames(data)))
  data$VAR <- data[,which(colnames(data) == variable)]
  data <- data[,-c(1:4)]
  return(data)
}



### list_val:  list results of Chi-square ---- 

list_val <- function(data, value){
  for(i in 1:(length(data))){
    tmp.dat <- data[[i]]
    tmp.dat <- tmp.dat[value]
    data [[i]] <- tmp.dat
  }
  data <- do.call(rbind.data.frame, data)
  return(data)
}



### add.par:   add parameters based on row names ---- 

add.par <- function(data){
  data$PER <- substr(rownames(data),1,3)
  data$ISO <- substr(rownames(data),4,4)
  return(data)
}



### analyze:   does main analysis ---- 

analyze <- function(an.dat, cum.dat){
  
  # classify annulus catchments
  c.an.dat <- an.dat
  an.class <- data.frame(class.min =seq(0,0.9,by=0.1), 
                         class.max =seq(0.1,1,by=0.1),
                         Amount = rep(NA,10), 
                         share = rep(NA,10))
  
  for(i in 1:length(c.an.dat)){
    an.class <- data.frame(class.min =seq(0,0.9,by=0.1), 
                           class.max =seq(0.1,1,by=0.1),
                           Amount = rep(NA,10), 
                           share = rep(NA,10))
    
    tmp.dat <- an.dat[[i]]
    for (i1 in 1:length(an.class$Amount)){
      an.class$Amount[i1] <- length(which((tmp.dat$H.Hmax > an.class$class.min[i1]) & 
                                            (tmp.dat$H.Hmax <= an.class$class.max[i1])))
    }
    an.class$share <- (an.class$Amount/sum(an.class$Amount))*100
    c.an.dat[[i]] <- an.class
  }
  
  
  # classify cumulative catchments
  c.cum.dat <- cum.dat
  cum.class <- data.frame(class.min =seq(0,0.9,by=0.1),
                          class.max =seq(0.1,1,by=0.1),
                          Amount = rep(NA,10),
                          share = rep(NA,10))
  
  for(i in 1:length(c.cum.dat)){
    cum.class <- data.frame(class.min =seq(0,0.9,by=0.1),
                            class.max =seq(0.1,1,by=0.1),
                            Amount = rep(NA,10),
                            share = rep(NA,10))
    
    tmp.dat <- cum.dat[[i]]
    for (i1 in 1:length(cum.class$Amount)){
      cum.class$Amount[i1] <- length(which((tmp.dat$H.Hmax >= cum.class$class.min[i1]) & 
                                             (tmp.dat$H.Hmax < cum.class$class.max[i1])))
    }
    
    cum.class$share <- (cum.class$Amount/sum(cum.class$Amount))*100
    c.cum.dat[[i]] <- cum.class
  }
  
  # ............... Calculate delta between site diversity and background diversity ...............
  
  # create a table with the relative amount by class for cumulative data and iterate it
  delta.cum.dat <- tabgen(c.cum.dat, "share")
  
  for (i in 1:length(delta.cum.dat)){
    tmp.dat <- delta.cum.dat[[i]]
    delta.cum.dat[[i]] <- sum(subset(tmp.dat$Background - tmp.dat$Sites ,
                                     tmp.dat$Background - tmp.dat$Site > 0))
  }
  tmp.dat <- names(delta.cum.dat)
  delta.cum.dat <- do.call(rbind.data.frame, delta.cum.dat)
  names(delta.cum.dat) <- "Delta"
  row.names(delta.cum.dat) <- tmp.dat
  
  delta.cum.dat <- add.par(delta.cum.dat)
  
  
  # create a table with the relative amount by class for annulus data and iterate it
  delta.an.dat <- tabgen(c.an.dat, "share")
  for (i in 1:length(delta.an.dat)){
    tmp.dat <- delta.an.dat[[i]]
    delta.an.dat[[i]] <- sum(subset(tmp.dat$Background - tmp.dat$Sites ,
                                    tmp.dat$Background - tmp.dat$Site > 0))
  }
  tmp.dat <- names(delta.an.dat)
  delta.an.dat <- do.call(rbind.data.frame, delta.an.dat)
  names(delta.an.dat) <- "Delta"
  row.names(delta.an.dat) <- tmp.dat
  
  delta.an.dat <- add.par(delta.an.dat)
  
  
  # ............... Dependence (Chi-square) between background diversity and site diversity ...............
  
  # generate cumulative Chi-square table
  chi.cum.dat <- tabgen(c.cum.dat, "Amount")
  
  # generate annuli Chi-square table
  chi.an.dat <- tabgen(c.an.dat, "Amount")
  
  
  # templates for classified Chi-square tables
  class.chi.cum.dat <-  chi.cum.dat
  class.chi.an.dat <- chi.an.dat
  
  # same templates for the expectation values
  class.chi.cum.exp <- class.chi.cum.dat
  class.chi.an.exp <- class.chi.an.dat
  
  
  # combine classes with expectation values below 1
  for(i in 1:(length(chi.cum.dat))){
    
    # class.chi{} needs: 
    # 1) the tested data and 
    # 2) a value to define at which expectation val. classes should be combined
    class.chi.cum.dat[[i]] <- class.chi(chi.cum.dat[[i]],1) 
  }
  for(i in 1:(length(chi.an.dat))){
    
    # class.chi{} needs:
    # 1) the tested data and 
    # 2) a value to define at which expectation val. classes should be combined
    class.chi.an.dat[[i]] <- class.chi(chi.an.dat[[i]],1)  
  }
  
  
  # return expectation values of combined classes to evaluate next step: 
  # classification by expectation values below 5
  for(i in 1:(length(chi.cum.dat))){
    class.chi.cum.exp[[i]] <- chisq.test(class.chi.cum.dat[[i]])
  }
  for(i in 1:(length(chi.an.dat))){
    class.chi.an.exp[[i]] <- chisq.test(class.chi.an.dat[[i]])
  }
  
  
  # cumulative catchments: 
  # combine classes if more than 20 % of the expectation values are still below 5
  for(i in 1:(length(class.chi.cum.dat))){
    tmp.dat <- class.chi.cum.exp[[i]]
    
    # Check if more than 20 % of the expectation values are less than 5
    if((length(which(tmp.dat$expected < 5)) / length(tmp.dat$expected)) > 0.20){ 
      
      # class.chi{} needs: 
      # 1)the tested data and 
      # 2) a value to define at which expectation value classes should be combined
      class.chi.cum.dat[[i]] <- class.chi(class.chi.cum.dat[[i]],5) 
    }
  }
  
  # annuli catchments: combine classes if more than 20 % of the expectation values are still below 5
  for(i in 1:(length(class.chi.an.dat))){
    tmp.dat <- class.chi.an.exp[[i]]
    
    # Check if more than 20% of the expectation values are less than 5
    if((length(which(tmp.dat$expected < 5)) / length(tmp.dat$expected)) > 0.20){ 
      
      # class.chi{} needs: 
      # 1)the tested data and 
      # 2) a value to define at which expectation val. classes should be combined
      class.chi.an.dat[[i]] <- class.chi(chi.an.dat[[i]],5) 
    }
  }
  
  
  # execute actual Chi-square Test
  for(i in 1:(length(class.chi.an.dat))){
    tmp.dat <- class.chi.an.dat[[i]]
    tmp.dat <- chisq.test(tmp.dat)
    chi.an.dat [[i]] <- tmp.dat
  }
  for(i in 1:(length(class.chi.cum.dat))){
    tmp.dat <- class.chi.cum.dat[[i]]
    tmp.dat <- chisq.test(tmp.dat)
    chi.cum.dat [[i]] <- tmp.dat
  }
  
  # create p-value table
  p_chi.an.dat <- list_val(chi.an.dat, 3)
  p_chi.cum.dat <- list_val(chi.cum.dat, 3)
  p_chi.an.dat <- add.par(p_chi.an.dat)
  p_chi.cum.dat <- add.par(p_chi.cum.dat)
  
  # create df-value table
  df_chi.an.dat <- list_val(chi.an.dat, 2)
  df_chi.cum.dat <- list_val(chi.cum.dat, 2)
  df_chi.an.dat <- add.par(df_chi.an.dat)
  df_chi.cum.dat <- add.par(df_chi.cum.dat)
  
  

  #............... Get amount of sites & background points ...............
  
  n.dat <- tabgen(c.cum.dat, "Amount")
  names(n.dat) <- substr(names(n.dat),1,3)
  n.dat <- n.dat[c(which(duplicated(names(n.dat)) == FALSE))]
  
  for (i in 1:length(n.dat)){
    n.dat[[i]] <- apply(n.dat[[i]],2,sum)
  }
  
  tmp.dat <- names(n.dat[[1]])
  tmp.dat2 <- names(n.dat)
  n.dat <- do.call(rbind.data.frame, n.dat)
  colnames(n.dat) <- tmp.dat
  row.names(n.dat) <- tmp.dat2

  
  
plot_dat <- list(c.an.dat, 
                 c.cum.dat, 
                 chi.an.dat, 
                 chi.cum.dat, 
                 delta.an.dat, 
                 delta.cum.dat, 
                 p_chi.an.dat, 
                 p_chi.cum.dat, 
                 df_chi.an.dat, 
                 df_chi.cum.dat, n.dat)

names(plot_dat) <- c("c.an.dat", 
                     "c.cum.dat", 
                     "chi.an.dat", 
                     "chi.cum.dat", 
                     "delta.an.dat", 
                     "delta.cum.dat", 
                     "p_chi.an.dat", 
                     "p_chi.cum.dat", 
                     "df_chi.an.dat", 
                     "df_chi.cum.dat", 
                     "n.dat")


return(plot_dat)  
  
}



### diachron_div.vs.div: plot the diversities of catchments vs the diversities of the landscape ---- 

diachron_div.vs.div <- function(eco.tmp, ecozone, plt.list, mode){

  
  # select eco.temp by annulus or by cumulative data, depending on mode
  
  if(mode == "Annulus"){
    eco.temp <- eco.temp[c(which(names(eco.temp) %like% "%an%"), which(names(eco.temp) == "n.dat"))]
    names(eco.temp) <- gsub('an.','',names(eco.temp)) 
  } else{
    eco.temp <- eco.temp[c(which(names(eco.temp) %like% "%cum%"),which(names(eco.temp) == "n.dat"))]
    names(eco.temp) <- gsub('cum.','',names(eco.temp)) 
  }

  
  # define names for facet plot
  PER.names <- t(eco.temp$n.dat)

  for(i in 1:(length(PER.names)/nrow(PER.names))){
    PER.names[1,i] <- paste0(colnames(PER.names)[i], 
                             "\n",
                             rownames(PER.names)[1], 
                             ": ", 
                             PER.names[1,i], 
                             "\n", 
                             rownames(PER.names)[2], 
                             ": ", 
                             PER.names[2,i])
  }
  PER.names <- PER.names[1,]
  rownames(PER.names) <- NULL  
  
  
  an.pltName <- c(paste0(ecozone, "_", mode))
  
 
  
  # Generating data plot and writing it into a list

  plt.list[[an.pltName]] <-  ggplot(eco.temp$con.dat, aes(CLASS, VAR)) +
                                            theme_bw() +
                                            theme(strip.text.y = element_text(angle = 0, face = "bold"),
                                                  strip.text.x = element_text(angle = 0, face = "bold", size = 7),
                                                  # get rid of minor grid
                                                  panel.grid.minor = element_blank())+ 
                                            geom_bar(data  = subset(eco.temp$con.dat, TYPE == "Site"), 
                                                     width = 0.5, 
                                                     alpha = 0.5 , 
                                                     stat="identity", 
                                                     fill="#c2b49a", 
                                                     colour="#244e5e") +
                                            geom_line(data = subset(eco.temp$con.dat, TYPE == "Back"), 
                                                      size = 1.2, 
                                                      alpha = 0.7,
                                                      linejoin = "round",
                                                      stat="identity",
                                                      colour="#244e5e") +
                                            geom_rect(mapping=aes(xmin = 0, xmax = 5, ymin = max(VAR)*0.65, ymax = max(VAR)), 
                                                      fill = "white", 
                                                      alpha = 0.05 ) +
                                            geom_text(data = eco.temp$delta.dat, 
                                                      aes(x = 0.5,  y = max(eco.temp$con.dat$VAR)*0.95,
                                                          label =paste("Delta =", round(Delta, digits = 2), "%"),
                                                          hjust = 0)) +
                                            geom_text(data = eco.temp$df_chi.dat,
                                                      aes(x = 0.5,  y = max(eco.temp$con.dat$VAR)*0.85,
                                                          label =paste("df =", parameter), hjust = 0)) +
                                            geom_text(data = eco.temp$p_chi.dat,
                                                      aes(x = 0.5,  y = max(eco.temp$con.dat$VAR)*0.75,
                                                          label = paste ( "p =", round(p.value, digits = 3)),
                                                          hjust = 0)) +
                                            scale_x_continuous(breaks = c(1:10))  +
                                            labs(title = "Diachronic Comparison: Catchment Diversity vs Landscape Diversity",
                                                 subtitle = paste0(mode, " catchments - Zone ",
                                                                   
                                                                   # if-loop is necessary, as "Europe" is not defined
                                                                   if(ecozone == main.zone){        
                                                                     "0"
                                                                   }
                                                                   else {
                                                                     which(eco.list == ecozone)
                                                                   },
                                                                   ": ", ecozone, if(impedance == T){
                                                                     ", Model B (including Groundcover)"
                                                                   }
                                                                   else {
                                                                     ", Model A (excluding Groundcover)"
                                                                   }),
                                                 y = "% of sites", x = "Shannon Diversity (H/Hmax*10)") +
                                            facet_grid( ISO ~ PER_F, labeller = labeller( PER_F = PER.names ))

  return(plt.list)

}



### synchron_div.vs.div: plot the diversities of catchments vs the diversities of the landscape ---- 

synchron_div.vs.div <- function(eco.temp, per, plt.list, mode){
  
  # select eco.temp by annulus or by cumulative data, depending on mode
  
  if(mode == "Annulus"){
    eco.temp <- eco.temp[c(which(names(eco.temp) %like% "%an%"), which(names(eco.temp) == "n.dat"))]
    names(eco.temp) <- gsub('an.','',names(eco.temp)) 
  } else{
    eco.temp <- eco.temp[c(which(names(eco.temp) %like% "%cum%"),which(names(eco.temp) == "n.dat"))]
    names(eco.temp) <- gsub('cum.','',names(eco.temp)) 
  }
  
  
  # define names for facet plot
  ECO.names <- t(eco.temp$n.dat)
  for(i in 1:(length(ECO.names)/4)){ # /4, as there are 4 rows
    ECO.names[1,i] <- paste0("Zone ", 
                             which(eco.list$Region == ECO.names["ECO",i]),
                             "\n",
                             rownames(ECO.names)[1],
                             ": ",
                             ECO.names[1,i],
                             "\n",
                             rownames(ECO.names)[2],
                             ": ",
                             ECO.names[2,i])
  }
  
  # add a col with NAs. This ensures, that a min. of 2 dimensions is given in the data frame, 
  # even if only one eco zone is given. (Other ways this has no effect)
  ECO.names <- cbind(ECO.names, rep(NA, 4))  
  colnames(ECO.names) <- ECO.names["ECO",]
  ECO.names <- ECO.names[-c(2:4),]
  rownames(ECO.names) <- NULL  
  
  
  an.pltName <- c(paste0(per, "_", mode))
  
  plt.list[[an.pltName]] <-  ggplot(eco.temp$con.dat, aes(CLASS, VAR)) +
                                  theme_bw() +
                                  theme(strip.text.y = element_text(angle = 0, face = "bold"),
                                        strip.text.x = element_text(angle = 0, face = "bold", size = 7),
                                        # get rid of minor grid
                                        panel.grid.minor = element_blank())+ 
                                  geom_bar(data  = subset(eco.temp$con.dat, TYPE == "Site"),
                                           width = 0.5,
                                           alpha = 0.5 ,
                                           stat="identity",
                                           fill="#c2b49a",
                                           colour="#244e5e") +
                                  geom_line(data = subset(eco.temp$con.dat, TYPE == "Back"),
                                            size = 1.2,
                                            alpha = 0.7 ,
                                            linejoin = "round",
                                            stat="identity",
                                            colour="#244e5e") +
                                  geom_rect(mapping=aes(xmin = 0, xmax = 10, ymin = max(VAR)*0.65, ymax = max(VAR)),
                                            fill = "white",
                                            alpha = 0.05 ) +
                                  geom_text(size = 3.5,
                                            data = eco.temp$delta.dat,
                                            aes(x = 0.5,
                                                y = max(eco.temp$con.dat$VAR)*0.95,
                                                label =paste("Delta =", round(Delta, digits = 2), "%"),
                                                hjust = 0)) +
                                  geom_text(size = 3.5,
                                            data = eco.temp$df_chi.dat,
                                            aes(x = 0.5,
                                                y = max(eco.temp$con.dat$VAR)*0.85,
                                                label =paste("df =", parameter),
                                                hjust = 0)) +
                                  geom_text(size = 3.5,
                                            data = eco.temp$p_chi.dat,
                                            aes(x = 0.5,  y = max(eco.temp$con.dat$VAR)*0.75,
                                                label = paste ( "p =", round(p.value, digits = 3)),
                                                hjust = 0)) +
                                  scale_x_continuous(breaks = c(1:10))+
                                  labs(title = "Synchronic Comparison: Catchment Diversity vs Landscape Diversity",
                                       subtitle = paste0(mode, " catchments - Period: ", per,
                                                         if(impedance == T){
                                                             ", Model B (including Groundcover)"
                                                           }
                                                           else {
                                                             ", Model A (excluding Groundcover)"
                                                           }
                                                        ),
                                       y = "% of sites", x = "Shannon Diversity (H/Hmax*10)") +
                                  facet_grid( ISO ~ ECO, labeller = labeller( ECO = ECO.names ))  

  
  return(plt.list)
  
}


### diachron_div.vs.iso: plot diversities for different isochones ---- 

diachron_div.vs.iso <- function(eco.dat, ecozone, plt.list, mode){


  # List to data.frame
  
  # get rid of Background data
  H.dat <- eco.dat[-which(names(eco.dat) %like% "%.b%")]
  H.dat <- do.call(rbind.data.frame, H.dat)
  H.dat$PER <- substr(rownames(H.dat),1,3)
  H.dat$ISO <- as.numeric(sub("\\D*(\\d{1}).*", "\\1", rownames(H.dat)))
  
  # delete useless data
  H.dat <- H.dat[,-(1:(which(names(H.dat) == "H.Hmax")-1))] 
  H.dat$NAME <- paste(H.dat$ISO, "h Catchment")

  # Define order of periods
  H.dat$PER <- factor(H.dat$PER, names(color.scheme)) 



  an.pltName <- c(paste0(ecozone,"_", mode))

  # Generating data plot and writing it into a list
  plt.list[[an.pltName]]  <- ggplot(H.dat, aes(NAME, H.Hmax, fill = PER)) +
                                            theme_bw()+
                                            theme(axis.text.x = element_text(angle = 60, vjust = 0.5),
                                                  panel.background = element_rect(fill = "transparent"), 
                                                  plot.background = element_rect(fill = "transparent", color = NA),
                                                  # get rid of minor grid
                                                  panel.grid.minor = element_blank(), 
                                                  legend.background = element_rect(fill = "transparent"),
                                                  legend.key = element_rect(fill = "transparent", colour = NA),
                                                  axis.line = element_line(colour = "black"))+ 
                                            geom_boxplot(alpha = 0.8) +
                                            scale_fill_manual(values = color.scheme)+
                                            labs(title = "Diachronic Comparison: Catchment Diversity by Isochrone",
                                              subtitle = paste0(mode, " catchments - Zone ", 
                                                                
                                                                # "if" loop is necessary, as "Europe" is not defined as 0
                                                                 if(names(valid.eco[zone]) == main.zone){ 
                                                                   "0"
                                                                 }
                                                                 else {
                                                                   which(eco.list == zone)
                                                                 },
                                                                 ": ", zone, if(impedance == T){
                                                                   ", Model B (including Groundcover)"
                                                                 }
                                                                else {
                                                                  ", Model A (excluding Groundcover)"
                                                                }),
                                               y = "Shannon diversity Index (H/Hmax)",
                                               x = "Isochrone per Period")
  
  return(plt.list)

}



### synchron_div.vs.iso: plot diversities for different isochones ---- 
synchron_div.vs.iso <- function(eco.dat, per, plt.list, mode){
  
  
  # List to data.frame
  
  # get rid of Background data
  H.dat <- eco.dat[-which(names(eco.dat) %like% "%.b%")]
  H.dat <- do.call(rbind.data.frame, H.dat)
  H.dat$PER <- substr(rownames(H.dat),1,3)
  H.dat$ISO <- as.numeric(sub("\\D*(\\d{1}).*", "\\1", rownames(H.dat)))
  H.dat$ECO <- H.dat$eco
  
  # delete useless data
  H.dat <- H.dat[,-(1:(which(names(H.dat) == "H.Hmax")-1))] 
  H.dat$NAME <- paste(H.dat$ISO, "h Catchment")
  
  # create data frame for main zone 
  main.H.dat <- H.dat
  main.H.dat$ECO <- main.zone
  H.dat <- rbind(H.dat, main.H.dat)
  
  # select Period / eco zone
  H.dat <- H.dat[which(H.dat$PER == per),]
  
  # select ECO
  H.dat <- H.dat[which(H.dat$ECO %in% names(per.valid.eco[[per]])),] 
  
  #Create an index-data frame to code the boxplot by eco region
  eco.code <- data.frame(eco = main.zone, code = 0)
  eco.code <- rbind(eco.code, 
                    data.frame(eco = as.character(c(names(per.valid.eco[[per]][-1]))), 
                               code = c(which(eco.list$Region %in% names(per.valid.eco[[per]])))))
  
  # to get rid of factors in $ECO
  H.dat$ECO <- as.character(H.dat$ECO) 
  
  H.dat$CODE <- NA
  for (i in 1: length(eco.code$eco)){
    index <- which(H.dat$ECO == eco.code$eco[i])
    H.dat$CODE[index] <- eco.code$code[i]
  }
  
  # Define order of boxplots
  # convert $ECO again to factors to define order
  H.dat$ECO <- with(H.dat, reorder(ECO, H.Hmax, median))  
  
  
  
  an.pltName <- c(paste0(per,"_", mode))
  
  # Generating data plot and writing it into a list
  plt.list[[an.pltName]]  <- ggplot(H.dat, aes(NAME, H.Hmax, fill = ECO)) +
    theme_bw()+
    theme(axis.text.x = element_text(angle = 60, vjust = 0.5),
          legend.title=element_text(size=12), 
          legend.text=element_text(size=8),
          panel.background = element_rect(fill = "transparent"), 
          plot.background = element_rect(fill = "transparent", color = NA), 
          panel.grid.minor = element_blank(), 
          legend.background = element_rect(fill = "transparent"), 
          legend.key = element_rect(fill = "transparent", colour = NA), 
          axis.line = element_line(colour = "black"))+
    geom_boxplot(alpha = 0.8) +
    scale_fill_manual(values = c(rep(color.scheme[[per]],
                                     length(eco.code$eco))),
                      labels= paste0("Zone ", eco.code$code,": ", eco.code$eco))+
    stat_summary(data = NULL,
                 geom = "text",
                 aes(group=ECO,
                     label=paste("Zone:",CODE),
                     fontface = "bold", y = 1),
                 hjust = 0.5,
                 angle = 90,
                 size = 2.5,
                 position = position_dodge(width=0.75)) +
    labs(title = "Synchronic Comparison: Catchment Diversity by Isochrone",
         subtitle = paste0(mode, " catchments - Period: ", per,
                           if(impedance == T){
                               ", Model B (including Groundcover)"
                             }
                             else {
                               ", Model A (excluding Groundcover)"
                             }
                           ),
         y = "Shannon diversity Index (H/Hmax)",
         x = "Isochrone per Period",
         fill = "Ecoregions")
  
  return(plt.list)
  
}




### do.CA:     custom function for correspondence analysis ---- 

do.CA <- function(plt.list, an.dat, eco.tmp){

  
# get site data in annulus-list, get rid of background data
s.an.dat <- an.dat[which(grepl("b", names(an.dat)) == F)]

#define temporary plot list for isochrones, to merge with original list afterwards
iso.tmp.plt.list <- list()

  # loop to iterate through isochrones
  for(iso in 1:4){
    
    # get data by isochrone in site-annulus-list
    s.an.dat.iso <- s.an.dat[grep(iso, names(s.an.dat))]
    
    # append period by data frame as a new column and merge to a single data frame
    df <- data.frame()
    for(i in 1:length(s.an.dat.iso)){
      s.an.dat.iso[[i]] <- s.an.dat.iso[[i]] %>% 
        mutate( PER = substr(names(s.an.dat.iso[i]), 1, 3))
      df <- rbind(s.an.dat.iso[[i]], df)
    }
    
    
    #filter for eco region, if function is not called for main.zone
    if(eco.tmp != main.zone){
      df <- df %>% filter(eco == eco.tmp)
    }
    
    
    # add a class for diversity (H.Hmax)
    an.class <- data.frame(class.min =seq(0,0.9,by=0.1),
                           class.max =seq(0.1,1,by=0.1))
    df$H.class <- NA
    for (i in 1: nrow(an.class)){
      df$H.class[which((df$H.Hmax > an.class$class.min[i]) &
                         (df$H.Hmax <= an.class$class.max[i]))] <- i
    }
    df$H.class[which((df$H.Hmax >= 0) & (df$H.Hmax <= 0.1))] <- 1
    
    
    # calculate median catchment composition within diversity classes
    median.dat <- df %>% 
      group_by(PER,H.class) %>% 
      summarise("Canyon" = median(LFC_1),
                "Valley In Plain" = median(LFC_2), 
                "Midslope Drainage" = median(LFC_3),
                "Upland Drainage" = median(LFC_4),
                "U-Valley" = median(LFC_5), 
                "Flat Plain" = median(LFC_6),
                "Sloping Plain" = median(LFC_7),
                "Flat Ridge-Top" = median(LFC_8), 
                "Ridge In Valley" = median(LFC_9),
                "Ridge In Plain" = median(LFC_10),
                "Midslope Ridge" = median(LFC_11),
                "Mountain Top" = median(LFC_12))
    
    median.dat[is.na(median.dat)==T] <- 0
    
    # c(...) creates a vector to select the columns that don't sum up to 0
    median.dat <- median.dat %>% select(c(1,2,which(colSums(median.dat[,3:14]) != 0)+2))
    
    
    
    # calculate Correspondence Analysis
    res.ca <- CA(median.dat[, 3:length(median.dat)], graph = FALSE)
    res.ca$row$PER <- median.dat$PER
    res.ca$row$H.class <- as.factor(median.dat$H.class)
    
    # write explained variance in plot list
    tmp.plt.list <- list(screeplot = fviz_screeplot(res.ca, addlabels = TRUE,
                                                 title = paste0("CA - ",
                                                                eco.tmp,
                                                                "\n",
                                                                iso,
                                                                "h Isochrone\nScree plot")),
                     Dim1 = fviz_contrib(res.ca,
                                         choice = "col",
                                         axes = 1, top = 12, 
                                         label = sort(round(res.ca$col$contrib[,1],digits = 1),
                                                      decreasing = T),
                                         title = paste0("CA - ",
                                                        eco.tmp,
                                                        "\n",
                                                        iso,
                                                        "h Isochrone\n
                                                        Contribution of Landforms to 1st Dimension")),
                     Dim2 = fviz_contrib(res.ca,
                                         choice = "col",
                                         axes = 2,
                                         top = 12, 
                                         label = sort(round(res.ca$col$contrib[,2], digits = 1),
                                                      decreasing = T),
                                         title = paste0("CA - ",
                                                        eco.tmp,
                                                        "\n",
                                                        iso,
                                                        "h Isochrone\n
                                                        Contribution of Landforms to 2nd Dimension")))
    
    
    # convert plot data to data frame for ggplot 
    # (https://rcrastinate.rbind.io/post/correspondance-analysis-ca-ggplot/)
    df.res.ca.row <- data.frame( Per = res.ca$row$PER, 
                                 Label = as.character(res.ca$row$H.class), 
                                 Dim1 = res.ca$row$coord[,1],
                                 Dim2 = res.ca$row$coord[,2], 
                                 Var = "H.Class", 
                                 n = count(df, PER, H.class)$n)
    df.res.ca.col <- data.frame( Per = "LFC",
                                 Label = colnames(median.dat)[3:length(median.dat)],
                                 Dim1 = res.ca$col$coord[,1],
                                 Dim2 = res.ca$col$coord[,2],
                                 Var = "LFC",
                                 # use n to define Size of LFC in plot
                                 n = 0.1) 
    
    df.res.ca <- rbind(df.res.ca.row,df.res.ca.col)
    
    df.res.ca <- mutate(df.res.ca,nPer = case_when(Per == "eup" ~ count(df, df$PER)[1,2],
                                                   Per == "gra" ~ count(df, df$PER)[2,2],
                                                   Per == "lgm" ~ count(df, df$PER)[3,2],
                                                   Per == "lpa" ~ count(df, df$PER)[4,2],
                                                   Per == "mag" ~ count(df, df$PER)[5,2]))
    
    df.res.ca[is.na(df.res.ca)] <- 1
    
    df.res.ca <- mutate(df.res.ca,nByPer = n / nPer)
    
    
    
    tmp.plt.list[["ca.plot"]] <- ggplot(df.res.ca,
                                        aes(Dim1, Dim2, color = Per,shape = Var, label = Label))+
      theme_minimal()+
      ggtitle(paste0("Correspondence analysis (CA)\n",
                     eco.tmp,
                     " - ",
                     iso,
                     "h Isochrone\nMedian catchment composition by diversity class"))+
      scale_shape(name = "Feature",
                  labels = c("Diversity Class", "Landform"))+
      scale_color_manual(name = "Period",
                         values = c(color.scheme,LFC = "dark red"),
                         labels = c(names(color.scheme), "[LFC]"))+
      scale_size(name = "% of Period")+
      geom_point(aes(size = nByPer*100), alpha = .65)+
      geom_text_repel(aes(label = Label),
                      max.overlaps = 75,
                      max.time = .5,
                      show.legend = F)+
      geom_vline(xintercept = 0, lty = "dashed") +
      geom_hline(yintercept = 0, lty = "dashed") +
      labs(x = paste("Dim 1 (",
                     round(tmp.plt.list$screeplot$data$eig[1],
                           digits = 1),
                     " %)"), 
           y = paste("Dim 2 (",
                     round(tmp.plt.list$screeplot$data$eig[2],
                           digits = 1),
                     " %)"))
    
    # add isochrone to plot names
    names(tmp.plt.list) <- paste(iso, names(tmp.plt.list), sep = "_")
    
    iso.tmp.plt.list <- c(iso.tmp.plt.list, tmp.plt.list)
    
  # end of iso iteration  
  }

# add eco zone to plot names
names(iso.tmp.plt.list) <- paste(eco.tmp, names(iso.tmp.plt.list), sep = "_")

plt.list <- c(plt.list, iso.tmp.plt.list)

return(plt.list)

}

