##
##
## Project: GEOPAL
##
##
## Script:  Landform diversity analysis
##
##
## Author:  Jonathan Schoenenberg
##
##
## Date:    2021-05-20
##
##
## Contact: j.schoenenberg@uni-koeln.de
##
##
##
### PROJECT SETUP ----
## gets the all you need to get the script running 
# --- --- --- --- --- --- --- --- --- --- --- --- ---


# Read packages
library(DescTools)
library(ggplot2) # plots
library(rstudioapi) # set working directory by document directory
library(dplyr) # data wrangling
library(ggrepel) # to avoid overlapping labels in plt.CA
library(fs)
library("FactoMineR") # calculates the CA
library("factoextra") # visualizes the contributions and scree plots 


# set working directory
setwd(dirname(getActiveDocumentContext()$path))


# source functions
source("GEOPAL_functions.R")


# ID for the main investigation area
zone = 0



### DATA SETUP  ----
## here you need to configure the data setup, 
## data is generated as described in Schoenenberg 2022, chapter 4.
## Background data without impedance are selected and 
## processed by the letters "na" in data names.
# --- --- --- --- --- --- --- --- --- --- --- --- ---

# Define the main investigation area
main.zone = "Europe"


# The color scheme defines the periods, as well as their chronological order!
# NOTE: KEEP THE SAME FORMAT
# 3 characters for period abbreviation 
color.scheme <- c(eup = "#f6e8c3",
                  gra = "#c2b49a",
                  lgm = "#dea12e",
                  mag = "#5d8a9d",
                  lpa = "#244e5e")


# Decide HERE which data to process.
# Make sure the folder containing the impeded data is called "impedance", 
# so that the getdata() function sets the variables right.
data <- getdata('Input/eu/impedance')

dat.list <- data$datlist


# this variable is needed for the names of the final .png files
impedance <- data$impedance


# get eco zones
# NOTE: THE ORDER OF THE LISTED REGIONS WILL DEFINE THE ORDER IN SYNCHRONOUSE PLOTS
setwd('Input/eu')
eco.list <- read.csv("ECO-List.csv", header = TRUE)

# convert list to vector
eco <- as.vector.factor(eco.list$Region) 


# order list by names
dat.list <- dat.list[order(names(dat.list))]

# order data, as the annuli are calculated by subtracting the next smaller 
# catchments for the whole dataframe
for(i in 1:length(dat.list)){
  tmp.dat <- dat.list[[i]]
  tmp.dat <- tmp.dat[order(tmp.dat$Site_ID),]
  dat.list[[i]] <- tmp.dat
}




### DATA PROCESSING - PART I  ----
## preparation of data for the main processing
# --- --- --- --- --- --- --- --- --- --- --- --- ---


# ...... Check for empty catchments and delete or warn ......


# in some cases input-data may contain a "NODATA" column caused by 
# methodological approach in QGIS.
# if the data is clean, these columns are deleted.
for(i in 1:length(dat.list)){
  dat.list[[i]] <- del.nodat(dat.list[[i]])
}



# ............... Convert pixels to square km ..............


for(i in 1:length(dat.list)){
  dat.list[[i]] <- sqm(dat.list[[i]])
}



# .................... Calculate annuli ...................


# group index for data relation by period
same.name <- gsub('.{1}$', '', names(dat.list))


# template for cumulative and annulus data
cum.dat <- dat.list
an.dat <- dat.list


# subtracting the cumulative data to create annuli
# 1st for: iterates related tables of the same period
for(i in 1:length(unique(same.name))){ 
  
  # gives an index on which tables belong together. 
  # ATTENTION: data must be sorted by isochrone for further processing!!
  index <- which(same.name == unique(same.name)[i])                 
  
  # 2nd for: iterates the tables
  for(i1 in 1:(length(index)-1)){                                   

    lfc <- which(colnames(cum.dat[[index[i1]]]) %like% "LFC%"==TRUE)
    per1 <- cum.dat[[index[i1]]]
    
    # i1+1 to get the next higher isochrone
    per2 <- cum.dat[[index[i1+1]]] 
    
    per2[min(lfc):max(lfc)] <- per2[min(lfc):max(lfc)] - per1[min(lfc):max(lfc)]
    an.dat[[index[i1+1]]] <- per2
  }
  
}



# ..... Calculate catchment extend & Shannon diversity .....


# extent of annulus catchments
for(i in 1: length(an.dat)){
  tmp.dat <- an.dat[[i]]
  lfc <- which(colnames(tmp.dat) %like% "LFC%"==TRUE)
  tmp.dat$Catchment_extend <- rowSums(tmp.dat[,c(lfc)])
  an.dat[[i]] <- tmp.dat
}


# extent of cumulative catchments
for(i in 1: length(cum.dat)){
  tmp.dat <- cum.dat[[i]]
  lfc <- which(colnames(tmp.dat) %like% "LFC%"==TRUE)
  tmp.dat$Catchment_extend <- rowSums(tmp.dat[,c(lfc)])
  cum.dat[[i]] <- tmp.dat
}


# create a copy with LFC-counts for correspondence analysis (CA),
# as original is overwritten during diversity calculation
CA.an.dat <- an.dat

# annulus Shannon diversity
for(i in 1: length(an.dat)){
  tmp.dat <- an.dat[[i]]
  tmp.dat[tmp.dat == 0] <- NA
  lfc <- which(colnames(tmp.dat) %like% "LFC%"==TRUE)
  tmp.dat[, c(lfc)] <- tmp.dat[, c(lfc)]/tmp.dat$Catchment_extend *
    log(tmp.dat[, c(lfc)]/tmp.dat$Catchment_extend)
  tmp.dat[is.na(tmp.dat)] <- 0
  tmp.dat$H.Hmax <- (rowSums(tmp.dat[,c(lfc)])*-1)/log(length(lfc))
  an.dat[[i]] <- tmp.dat
  CA.an.dat[[i]]$H.Hmax <- tmp.dat$H.Hmax
}


# cumulative Shannon diversity
for(i in 1: length(cum.dat)){
  tmp.dat <- cum.dat[[i]]
  tmp.dat[tmp.dat == 0] <- NA
  lfc <- which(colnames(tmp.dat) %like% "LFC%"==TRUE)
  tmp.dat[, c(lfc)] <- tmp.dat[, c(lfc)]/tmp.dat$Catchment_extend * 
    log(tmp.dat[, c(lfc)]/tmp.dat$Catchment_extend)
  tmp.dat[is.na(tmp.dat)] <- 0
  tmp.dat$H.Hmax <- (rowSums(tmp.dat[,c(lfc)])*-1)/log(length(lfc))
  cum.dat[[i]] <- tmp.dat
}



### DATA PROCESSING - PART II  ----
## main part of the program
# --- --- --- --- --- --- --- --- --- --- --- --- ---


#.......... Iterate by ecoregion and periods ..........

# define lists for valid and invalid ecoregions
per.valid.eco <- list()

# an ecoregion is considered invalid if no valid Chi-square test between 
# the background distribution and site distribution is possible 
# OR 
# if more than one degree of freedom by ecoregion and throughout the isochrones is <= 1 (in annulus)
per.invalid.eco <- list()


# iterate by period
for(p in 1: length(color.scheme)){ 

  tmp.an.dat <- an.dat[names(an.dat) %like% paste0(names(color.scheme[p]), "%")]
  tmp.cum.dat <- cum.dat[names(cum.dat) %like% paste0(names(color.scheme[p]), "%")]

  invalid.eco <- c()
  valid.eco <- list()
  
  # calculate (do the main analysis) once without eco filter
  final.dat <-  analyze(tmp.an.dat, tmp.cum.dat) 
      
      # calculate data for site diversity vs landscape diversity
      # 2nd argument in Function for name of the desired data column
      final.dat$con.an.dat <- con.dat(final.dat$c.an.dat, "share")    
      final.dat$con.cum.dat <- con.dat(final.dat$c.cum.dat, "share")
      
      # append ecoregion to data 
      for(i in 5: length(final.dat)){
        final.dat[[i]][[length(final.dat[[i]])+1]] <- c(rep(zone, nrow(final.dat[[i]])))
        names(final.dat[[i]])[[length(final.dat[[i]])]] <-  "ECO"
      }
      
      # append period to data
      # add a column "PER" to n.dat (needed for Selection by Period)
      final.dat$n.dat$PER <- substr(rownames(final.dat$n.dat),1,3)


      valid.eco[[main.zone]] <- final.dat
   
  
  # iterate by eco zone
  for(e in 1: nrow(eco.list)){
    zone <- eco[e]
    
    # once for annulus data ...
    eco.an.dat <- tmp.an.dat
    for(i in 1:(length(tmp.an.dat))){
      tmp.dat <- tmp.an.dat[[i]]
      tmp.dat <- tmp.dat[tmp.dat$eco == zone, ]
      eco.an.dat[[i]] <- tmp.dat
    }

    # ... and once for cumulative data
    eco.cum.dat <- tmp.cum.dat
    for(i in 1:(length(tmp.cum.dat))){
      tmp.dat <- tmp.cum.dat[[i]]
      tmp.dat <- tmp.dat[tmp.dat$eco == zone, ]
      eco.cum.dat[[i]] <- tmp.dat
    }


    # check if a valid analysis of the eco zone is possible (see below)
    final.dat <- try(analyze(eco.an.dat, eco.cum.dat))


    # if eco zone yields no valid Chi-square test OR more than one degree of freedom <= 1 (in annulus), 
    # skip to next iteration
    if(isTRUE(class(final.dat)=="try-error") || sum(final.dat$df_chi.an.dat$parameter <= 1) > 1) { 
      
      # store invalid eco zone
      invalid.eco <- append(invalid.eco, eco[e]) 
      next
      }
    else {
      valid.eco[[zone]] <- final.dat
      
      # calculate data for site diversity vs landscape diversity
      # 2nd argument in Function for name of the desired data column
      valid.eco[[zone]]$con.an.dat <- con.dat(valid.eco[[zone]]$c.an.dat, "share")    
      valid.eco[[zone]]$con.cum.dat <- con.dat(valid.eco[[zone]]$c.cum.dat, "share")
      
      # append eco zone to data
      for(i in 5: length(valid.eco[[zone]])){
        valid.eco[[zone]][[i]][[length(valid.eco[[zone]][[i]])+1]] <- c(rep(zone, nrow(valid.eco[[zone]][[i]])))
        names(valid.eco[[zone]][[i]])[[length(valid.eco[[zone]][[i]])]] <-  "ECO"
      }
      
      # append period to data by adding a column "PER" to n.dat (needed for Selection by Period)
      valid.eco[[zone]]$n.dat$PER <- substr(rownames(valid.eco[[zone]]$n.dat),1,3)
      
    }

  }
  
  per.invalid.eco[[names(color.scheme[p])]] <- invalid.eco
  per.valid.eco[[names(color.scheme[p])]] <- valid.eco

}


# define which zones occur in each period as diachronic regions
# create a matrix of periods by ecoregion:
diachronicon <- data.frame(matrix(ncol = length(eco.list$Region), nrow = length(names(color.scheme))), 
                           row.names = names(color.scheme)) 
names(diachronicon) <- eco.list$Region
diachronicon[main.zone] <- rep(NA, nrow(diachronicon))


# fill this matrix with "1" if eco zones yielded valid results 
for(i in 1:length(per.valid.eco)){
  diachronicon[i, which(names(diachronicon) %in% names(per.valid.eco[[i]]))] <- 1
}


# define the names of diachronic eco zones by defining which region occurs in each period
dia.eco <- names(diachronicon[which(colSums(diachronicon) == length(names(color.scheme)))])  


# relate names of diachronic ecoregions with the data of diachronic ecoregions
tmp.dia.dat <- list()
for(i in 1:length(per.valid.eco)){
  tmp.dia.dat[[names(per.valid.eco[i])]] <- per.valid.eco[[i]][which(names(per.valid.eco[[i]]) %in% dia.eco)]
}


# merge data for diachronic analysis
# define list to merge data in
dia.dat <- tmp.dia.dat[[1]]

  # start with 2 as first is already stored in dia.dat
  for (p in 2: length(color.scheme)) {    
    
    # iterate by eco
    for(i in 1:length(dia.eco)){          
      
      # first for positions are lists and need to be merged differently from the remaining data frames
      for (d1 in 1:4) {                   
        dia.dat[[i]][[d1]] <-  c(dia.dat[[i]][[d1]],tmp.dia.dat[[p]][[i]][[d1]])    
      }
      
      # first for positions are lists and need to be merged differently from the remaining data frames
      for (d2 in 5:13) {                  
        dia.dat[[i]][[d2]] <-  rbind(dia.dat[[i]][[d2]],tmp.dia.dat[[p]][[i]][[d2]])
      }
  
    }
    
  }
  


### GENERATING PLOTS  ----
## 
# --- --- --- --- --- --- --- --- --- --- --- --- ---  

#.......... Diachronic Plot: Site Diversity vs Landscape Diversity ..........

# define liste to store diachonic plots in
diachronic.pltlist1 <- list()



# iterate by zone
for (e in 1: length(dia.dat)){
  
  # create temporal variable for eco iteration
  eco.temp <- dia.dat[e][[1]]
  zone <- names(dia.dat[e])
  
  
  # adds a row with period factors "PER_F" for correct display of chronological order
  for(i in 5:length(eco.temp)){
    eco.temp[[i]]$PER_F <- factor(eco.temp[[i]]$PER, levels = names(color.scheme))
  }
  
  
  diachronic.pltlist1 <- diachron_div.vs.div(eco.temp, zone, diachronic.pltlist1, "Annulus")  
  diachronic.pltlist1 <- diachron_div.vs.div(eco.temp, zone, diachronic.pltlist1, "Cummulative")

  
}


#.......... Synchronic Plot: Site Diversity vs Landscape Diversity ..........

tmp.sync.dat <- per.valid.eco


# merge data for synchronic analysis
## define list to merge data in
sync.dat <- vector(mode = "list", length = length(per.valid.eco))
names(sync.dat) <- c(names(per.valid.eco) )            

# iterate by period
for (p in 1: length(sync.dat)) {                
  
  # write data from second ecoregion as sample. skip the first one as the main zone is not needed in synchronic comparison
  sync.dat[p] <- list(per.valid.eco[[p]][[2]][5:13]) 
  
  # iterate by eco
  for(e in 3:length(per.valid.eco[[p]])){       
    
    # first for positions are lists and need to be merged differently from the remaining data frames
    for (d2 in 1:9) {                          
      sync.dat[[p]][[d2]] <- rbind(sync.dat[[p]][[d2]],per.valid.eco[[p]][[e]][[d2+4]])
    }
    
  }
  
}



# define list to store synchronic plots in
synchronic.pltlist1 <- list()

for(p in 1: length(sync.dat)){
  per <- names(sync.dat[p])
  
  synchronic.pltlist1 <- synchron_div.vs.div(sync.dat[[p]], per, synchronic.pltlist1, "Annulus") 
  synchronic.pltlist1 <- synchron_div.vs.div(sync.dat[[p]], per, synchronic.pltlist1, "Cummulative") 
}


#.......... Diachronic Plot: Site Diversity vs Isochrones ..........

# create liste to stroe plots in
diachronic.pltlist2 <- list()
zone <- main.zone

# calculate once without eco filter
diachronic.pltlist2 <- diachron_div.vs.iso(an.dat, zone, diachronic.pltlist2, "Annulus")  
diachronic.pltlist2 <- diachron_div.vs.iso(cum.dat, zone, diachronic.pltlist2, "Cummulative")  




eco.an.dat <- an.dat
eco.cum.dat <- cum.dat

# Iterate by Eco-zone
# [-1] as "Europe" is not listed among the original eco zones
for(e in 1: length(dia.dat[-1])){               
  
  # +1 to skip "Europe", see above
  zone <- names(dia.dat[e+1])                   

  # selecting the different eco zones for annulus... 
  for(i in 1:(length(an.dat))){                   
    tmp.dat <- an.dat[[i]]
    tmp.dat <- tmp.dat[tmp.dat$eco == zone, ]
    eco.an.dat[[i]] <- tmp.dat
  }
  
  eco.cum.dat <- cum.dat
  
  # ...and cummulative data
  for(i in 1:(length(cum.dat))){                  
    tmp.dat <- cum.dat[[i]]
    tmp.dat <- tmp.dat[tmp.dat$eco == zone, ]
    eco.cum.dat[[i]] <- tmp.dat
  }
  
  # calculate plots (filtered by eco) and store them in an list
  diachronic.pltlist2 <- diachron_div.vs.iso(eco.an.dat, zone, diachronic.pltlist2, "Annulus")
  diachronic.pltlist2 <- diachron_div.vs.iso(eco.cum.dat, zone, diachronic.pltlist2, "Cummulative")

}

  


#.......... Synchronic Plot: Site Diversity vs Isochrones ..........

# create list to store plots in
synchronic.pltlist2 <- list()


for(per in 1: length(sync.dat)){
  synchronic.pltlist2 <- synchron_div.vs.iso(an.dat, names(color.scheme)[per], synchronic.pltlist2, "Annulus")
  synchronic.pltlist2 <- synchron_div.vs.iso(cum.dat, names(color.scheme)[per], synchronic.pltlist2, "Cummulative")
}




#.......... CA (Correspondence Analysis) ..........

plt.CA <- list()

# once for the whole data set
plt.CA <- do.CA(plt.CA, CA.an.dat, main.zone)

# once for every main ecological region
# dia.dat contains valid ecoregions for the diachronic comparison, -1 to skip "Europe"
for( e in names(dia.dat)[-1]){ 

  plt.CA <- do.CA(plt.CA, CA.an.dat, e)
  
} 


### SAVING DATA  ----
## saving plots to .png
# --- --- --- --- --- --- --- --- --- --- --- --- ---

# back to root
setwd(dirname(getActiveDocumentContext()$path))


# create folder structure for Output
if(dir.exists('Output') == F){
  dir.create('Output')
  setwd('Output')
  dir.create('Diachronic Comparison')
  dir.create('Synchronic Comparison')
  dir.create('CA')
} else {
  setwd('Output')
}


# diachronic comparison
setwd('Diachronic Comparison')

pltlist <- c(diachronic.pltlist1, diachronic.pltlist2)

for (i in 1: length(pltlist)){
  plot <- pltlist[[i]]
  data.name <- paste0(if(impedance == F){
                        "Model A_"
                        } else{
                        "Model B_"
                      },
                      strsplit(plot$labels$title , ": ")[[1]][[2]],
                      "_",
                      
                      # combine text following the ":" in the title with the subtitle where ": is replaced by ","
                      gsub(":", ",", plot$labels$subtitle ) 
                      )
  
  # cut the name, path plus name should be below 264 characters.
  data.name <- strsplit(data.name,",")[[1]][1]
  
  ggsave(filename = paste0(data.name, ".png"),
         plot = plot,
         device = "png",
         dpi = 300,
         width = 27,
         height = 18,
         units = "cm",
         limitsize = T)
}
setwd('..')


# Synchronic comparison
setwd('Synchronic Comparison')

pltlist <- c(synchronic.pltlist1, synchronic.pltlist2 )

for (i in 1: length(pltlist)){
  plot <- pltlist[[i]]
  data.name <- paste0(if(impedance == F){
                          "Model A_"
                        } else{
                          "Model B_"
                      },
                      strsplit(plot$labels$title , ": ")[[1]][[2]],
                      "_",
                      
                      strsplit(plot$labels$subtitle, " - " )[[1]][1],
                      "_",
                      
                      plot$data$PER[1]
                      )
  
  
  ggsave(filename = paste0(data.name, ".png"),
         plot = plot, device = "png",
         dpi = 300, width = 27,
         height = 18,
         units = "cm",
         limitsize = T)
}
setwd('..')


# CA
setwd('CA')

pltlist <- plt.CA

for (i in 1: length(pltlist)){
  plot <- pltlist[[i]]
  data.name <- paste0(if(impedance == F){
                        "Model A_"
                      } else{
                        "Model B_"
                    },
                    names(pltlist)[i]
                    )
  ggsave(filename = paste0(data.name, ".jpg"),
         plot = plot, device = "jpg",
         dpi = 300,
         limitsize = T)
}




paste("The following eco region provided no significant result:", sort(invalid.eco))


